/*
 * softGL - A software based graphics library for PS3
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PS3SDK root for details.
 *
 * font.h - Prototypes for Bitmap Font
 *
 * Copyright (c) 2010 Ifcaro <taslokooo@gmail.com>
 *
 * Based on NFont (c) Jonathan Dearborn
 *
 */
 
#ifndef _NFONT_H__
#define _NFONT_H__

typedef struct NFont TFont;

typedef struct NFontAnim_Data
{
    struct NFont* font;
    
    TImage* dest;
    TImage* src;
    char* text;  // Buffer for efficient drawing
    int height;
    int* charPos;
    int* charWidth;
    int maxX;
    
    int index;
    int letterNum;
    int wordNum;
    int lineNum;
    int startX;
    int startY;
    void* userVar;
} NFontAnim_Data;

struct NFont
{
    TImage* src;  // bitmap source of characters
    TImage* dest; // Destination to blit to
    uint8_t cleanUp;

    int height;

    int maxWidth;
    int baseline;
    int ascent;
    int descent;

    int lineSpacing;
    int letterSpacing;

    int charPos[256];
    int charWidth[256];
    int maxPos;

    char* buffer;  // Buffer for efficient drawing

    struct NFontAnim_Data data;  // Data is wrapped in a struct so it can all be passed to 
                          // the function pointers

};


// Function pointer
typedef void (*NFontAnim_Fn)(int*, int*, NFontAnim_Data*);

#ifdef __cplusplus
extern "C" {
#endif

void NF_Init(TFont* font);
TFont *NF_New();
void NF_Free(TFont* font);
void NF_Push(TFont* font);
TFont* NF_Pop();

TImage* NF_GetDest();
void NF_SetSpacing(int LetterSpacing);
int NF_GetSpacing();
void NF_SetLineSpacing(int LineSpacing);
int NF_GetLineSpacing();
int NF_GetBaseline();
int NF_GetMaxWidth();
void NF_SetBuffer(unsigned int size);
void NF_SetCleanUp(uint8_t enable);
void NF_SetDest(TImage* Dest);
uint8_t NF_ResetFont(TImage* Dest, TImage* FontSurface, uint8_t CleanUp);
uint32_t NF_GetPixel(TImage *Surface, int x, int y);

TImage* NF_NewColorSurface(uint32_t top, uint32_t bottom, int heightAdjust);

void NF_DrawToSurface(int x, int y, const char* text);
char* NF_CopyString(const char* c);
void NF_Draw(int x, int y, const char* formatted_text, ...);
void NF_DrawCenter(int x, int y, const char* formatted_text, ...);
void NF_DrawRight(int x, int y, const char* formatted_text, ...);
int NF_GetHeight(const char* formatted_text, ...);
int NF_GetWidth(const char* formatted_text, ...);
uint8_t NF_SetFont(TImage* FontSurface, uint8_t CleanUp);


int NF_SetBaseline(int Baseline);
int NF_GetAscentChar(const char character);
int NF_GetAscent(const char* formatted_text, ...);
int NF_GetDescentChar(const char character);
int NF_GetDescent(const char* formatted_text, ...);

void NF_DrawPos(int x, int y, NFontAnim_Fn posFn, const char* text, ...);
void NF_DrawPosX(int x, int y, NFontAnim_Fn posFn, void* userVar, const char* text, ...);
void NF_DrawAll(int x, int y, NFontAnim_Fn allFn, const char* text, ...);
void NF_DrawAllX(int x, int y, NFontAnim_Fn allFn, void* userVar, const char* text, ...);

#ifdef __cplusplus
}
#endif

#endif // _NFONT_H__
