/*
 * softGL - A software based graphics library for PS3
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PS3SDK root for details.
 *
 * gfx.h - Protorypes for Graphics Functions
 *
 * Copyright (c) 2010 Ifcaro <taslokooo@gmail.com>
 */

#include <ps3av.h>
#include <ps3gpu.h>
#include <ps3mmu.h>
#include <ps3console.h>

static const struct {
  uint8_t vmode;
  uint8_t aspect;
  uint16_t width;
  uint16_t height;
} modeTab[] = {
  [PS3AV_MODE_480I] =       {  1, PS3AV_ASPECT_4_3 ,  720,  480 },
  [PS3AV_MODE_480P] =       {  5, PS3AV_ASPECT_4_3 ,  720,  480 },
  [PS3AV_MODE_720P_60HZ] =  {  9, PS3AV_ASPECT_4_3 , 1280,  720 },
  [PS3AV_MODE_1080I_60HZ] = {  7, PS3AV_ASPECT_16_9, 1920, 1080 },
  [PS3AV_MODE_1080P_60HZ] = { 11, PS3AV_ASPECT_16_9, 1920, 1080 },
  [PS3AV_MODE_576I] =       {  3, PS3AV_ASPECT_4_3 ,  720,  576 },
  [PS3AV_MODE_576P] =       {  6, PS3AV_ASPECT_4_3 ,  720,  576 },
  [PS3AV_MODE_720P_50HZ] =  { 10, PS3AV_ASPECT_4_3 , 1280,  720 },
  [PS3AV_MODE_1080I_50HZ] = {  8, PS3AV_ASPECT_16_9, 1920, 1080 },
  [PS3AV_MODE_1080P_50HZ] = { 12, PS3AV_ASPECT_16_9, 1920, 1080 },
  [PS3AV_MODE_WXGA] =       { 13, PS3AV_ASPECT_16_9, 1280,  768 },
  [PS3AV_MODE_SXGA] =       { 14, PS3AV_ASPECT_4_3 , 1280, 1024 },
  [PS3AV_MODE_WUXGA] =      { 15, PS3AV_ASPECT_16_9, 1920, 1200 }
};

typedef
struct TImage
{
	int width;
	int height;
	void *ptr;
} TImage;

typedef
struct TRect
{
	int x;
	int y;
	int w;
	int h;
} TRect;

#include "font.h"

#define DrawText NF_Draw
#define DrawTextCenter NF_DrawCenter
#define DrawTextRight NF_DrawRight
#define DrawTextPos NF_DrawPos
#define DrawTextPosX NF_DrawPosX
#define DrawTextAll NF_DrawAll
#define DrawTextAllX NF_DrawAllX

TImage screen;

// System

void InitGFX(uint32_t mode);
void Flip();
uint32_t GetTicks();

// Loading functions

TFont *LoadFont(TImage *image);
TImage *LoadImageRaw(void *src, int width, int height);

// Primitives

void DrawPoint(uint32_t color, int px, int py);
void DrawLine(uint32_t color, int x0, int y0, int x1, int y1);
void DrawRect(uint32_t color, int px, int py, int w, int h);
void DrawCircle(uint32_t color, int xCenter, int yCenter, int radius);
void DrawCircleFill(uint32_t color, int xCenter, int yCenter, int radius);
void Cls(uint32_t color);

// Images

void DrawImage(TImage *image, int px, int py, int w, int h);
void BlitImage(TImage *imgsrc, TRect *srcrect, TImage *imgdst, TRect *dstrect);
uint32_t GetPixel(TImage *image, int x, int y);

