/*
 * softGL - A software based graphics library for PS3
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PS3SDK root for details.
 *
 * Copyright (c) 2010 Ifcaro <taslokooo@gmail.com>
 */

#include <stdio.h>
#include <string.h>
#include "gfx.h"

#include "math.h"

#define PI 3.1416

#include "image.h"
#include "fuente.h"

void swirly(int *x, int *y, NFontAnim_Data *data)
{
    int t = GetTicks();
    *x += (int)(data->wordNum*5*cos(t/500.0));
    *y += (int)(data->wordNum*20*sin(t/500.0));
}

void spirally(int *x, int *y, NFontAnim_Data *data)
{
    int t = GetTicks();
    
    // Negate auto-placement
    *x = data->startX;
    *y = data->startY;
    
    float place = (float)(data->index + 1) / strlen(data->text);
    *x += (int)(80*(place + .7)*cos(place * 2*PI - t/1000.0));
    *y += (int)(40*(place + .7)*sin(place * 2*PI - t/1000.0));
}

void wavy(int *x, int *y, NFontAnim_Data *data)
{
    int t = GetTicks();
    *y += (int)(20*sin(t/250.0 + data->index/4.0));
}

void stretchy(int *x, int *y, NFontAnim_Data *data)
{
    int t = GetTicks();
    
    float place = (float)(data->index) / strlen(data->text);
    *x += (int)(80*(place - 0.5)*cos(t/500.0));
}

int main(int argc, char *argv[])
{
	int i=0, isign=1, j=0, jsign=1;
	
	InitGFX(PS3AV_MODE_480P);
	
	TImage *image = LoadImageRaw(image_raw, 128, 128);
	TImage *img_fuente = LoadImageRaw(fuente_raw, 3711, 25);

	LoadFont(img_fuente);

	while(1){
		DrawCircleFill(0xffff00ff, 360, 240, i);	
		DrawImage(image, 200-(i/2), 200-(i/2), i, i);
		DrawRect(0x0000ffff, 520-(i/2), 200-(i/2), i, i);
		DrawLine(0xff0000ff, 0,0,j,screen.height);
		DrawLine(0x00ff00ff, screen.width,0,screen.width-j,screen.height);
		
		if(isign==1){
			if(i<256){
				i++;
			}else{
				isign=-1;
			}
		}else{
			if(i>0){
				i--;
			}else{
				isign=1;
			}
		}
		if(jsign==1){
			if(j<720){
				j++;
			}else{
				jsign=-1;
			}
		}else{
			if(j>0){
				j--;
			}else{
				jsign=1;
			}
		}
        DrawTextPos(270, 50, (NFontAnim_Fn)stretchy, "softGL Demo!");
		DrawTextPos(550, 170, (NFontAnim_Fn)spirally, "Version 0.1");
		DrawTextPos(50, 170, (NFontAnim_Fn)swirly, "It has zero optimization");
		DrawTextPos(80, 370, (NFontAnim_Fn)wavy, "This demo never ends, you have to turn off the ps3.");
		DrawText(340, 450, "Visit http://ps3zone.ifcaro.net");
		Flip();
		Cls(0);
	}

    return 0;
}
